/** This file is part of PeerHood.
*
*   PeerHood is free software: you can redistribute it and/or modify
*   it under the terms of the GNU Lesser General Public License 
*   version 2 as published by the Free Software Foundation.
*
*   PeerHood is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU Lesser General Public License for more details.
*
*   You should have received a copy of the GNU Lesser General Public
*   License along with PeerHood. If not, see <http://www.gnu.org/licenses/>.
*/

// Emacs, hail the mighty -*-C++-*-

/**
 * Copyright 2003 LUT. .
 *
 * @name WLANCreator.h
 * @memo WLAN-specific implementation of the MAbstractCreator interface.
 *
 * @version 0.1
 * date     30.06.2003
 * change   30.06.2003
 */

#ifndef __WLANCREATOR_H__
#define __WLANCREATOR_H__

#include <AbstractCreator.h>

/**
 * @memo WLAN-specific implementation of the MAbstractCreator interface.
 * This class is used by the PeerHood factory to create connection objects that
 * implement the MAbstractConnection interface. 
 * @doc WLAN-specific implementation of the MAbstractCreator interface. 
 * This class is used by the PeerHood factory to create connection objects that
 * implement the MAbstractConnection interface. The idea is to provide a 
 * transparent way to create objects with specific properties. This makes the
 * whole PeerHood system quite flexible; new transport medium can be added 
 * without any modifications to the source code itself.
 *
 * @see MAbstractCreator
 * @see MAbstractConnection
 * @see TCPConnection
 * @see Factory
 */
class CWLANCreator : public MAbstractCreator
{
 public:
  CWLANCreator();
  MAbstractConnection* CreateConnectionL(const std::string& aProto);
  MAbstractPinger* CreatePingerL(const std::string& aProto, const std::string& aAddress);
  MAbstractMonitor* CreateMonitorL(const std::string& aProto, const std::string& aAddress);
  const std::string& GetPrototype();

 private:
  std::string iProto;
  bool LoadParameter(std::string& aParameter);
};

#endif // __WLANCREATOR_H__
